#include "Theme.h"
#include <QtCore/QLibraryInfo>
#include <QtCore/QStandardPaths>
#include <QtGui/QGuiApplication>
namespace Dekko
{
namespace Themes
{

QString findThemeFile() {
    QString configFile;
    // TODO: make this configurable
    const QString filePath = QLatin1String("configuration/styles.conf");
    QStringList paths = QStandardPaths::standardLocations(QStandardPaths::DataLocation);
    paths.prepend(QDir::currentPath());
    paths.prepend(QCoreApplication::applicationDirPath());
    Q_FOREACH (const QString &path, paths) {
        QString myPath = path + QLatin1Char('/') + filePath;
        if (QFile::exists(myPath)) {
            configFile = myPath;
            break;
        }
    }
    if (!configFile.isEmpty()) {
        qDebug() << "Theme file found";
        return configFile;
    }
    qDebug() <<" Theme file not found";
    return QString();
}

Theme::Theme(QObject *parent) :
    QObject(parent)
{
    m_theme = findThemeFile();
    if (m_theme.isEmpty()) {
        qFatal("No theme file found :-/");
    }
    m_common = new Themes::Common(this, m_theme);
    m_actions = new Themes::Actions(this, m_theme);
    m_avatar = new Themes::Avatar(this, m_theme);
    m_colors = new Themes::Colors(this, m_theme);
    m_buttons = new Themes::Buttons(this, m_theme);
    m_header = new Themes::Header(this, m_theme);
    emit themeChanged();
}

QObject *Theme::common() const
{
    return m_common;
}

QObject *Theme::header() const
{
    return m_header;
}

QObject *Theme::actions() const
{
    return m_actions;
}

QObject *Theme::avatar() const
{
    return m_avatar;
}

QObject *Theme::colors() const
{
    return m_colors;
}

QObject *Theme::buttons() const
{
    return m_buttons;
}

QString Theme::currentTheme() const
{
    return m_theme;
}

QObject *Theme::factory(QQmlEngine *engine, QJSEngine *scriptEngine)
{
    Q_UNUSED(scriptEngine);
    Theme *theme = new Theme(reinterpret_cast<QObject*>(engine));
    return theme;
}
}
}
