#ifndef THEME_H
#define THEME_H

#include <QObject>
#include <QSettings>
#include <QtQuick>
#include <QQmlEngine>
#include <QJSEngine>
#include <QtGui/QGuiApplication>
#include "Styles.h"

namespace Dekko
{
namespace Themes
{
class Styles;

class Theme : public QObject
{
    Q_OBJECT
    Q_PROPERTY(QObject *common READ common NOTIFY themeChanged)
    Q_PROPERTY(QObject *header READ header NOTIFY themeChanged)
    Q_PROPERTY(QObject *actions READ actions NOTIFY themeChanged)
    Q_PROPERTY(QObject *avatar READ avatar NOTIFY themeChanged)
    Q_PROPERTY(QObject *colors READ colors NOTIFY themeChanged)
    Q_PROPERTY(QObject *buttons READ buttons NOTIFY themeChanged)
    Q_PROPERTY(QString currentTheme READ currentTheme NOTIFY themeChanged)
//    Q_PROPERTY(QStringList installedThemes READ installedThemes NOTIFY installedThemesChanged)

public:
    explicit Theme(QObject *parent = 0);

    QObject *common() const;
    QObject *header() const;
    QObject *actions() const;
    QObject *avatar() const;
    QObject *colors() const;
    QObject *buttons() const;

    QString currentTheme() const;
//    void setCurrentTheme(const QString &themeName);

//    QStringList installedThemes() const; // List of installed themes

    static QObject *factory(QQmlEngine *engine, QJSEngine *scriptEngine);

signals:
    void themeChanged();
//    void installedThemesChanged();

private:
    Themes::Common *m_common;
    Themes::Actions * m_actions;
    Themes::Avatar *m_avatar;
    Themes::Colors *m_colors;
    Themes::Buttons *m_buttons;
    Themes::Header *m_header;
    QString m_theme;

};
}
}
#endif // THEME_H
